#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
using SharpDX;
using SharpDX.Direct2D1;

#endregion

namespace NinjaTrader.NinjaScript.Indicators {

	public class TrendLineOne : Indicator {
		
		// Widerstandslinie
		private int x_h1 = 0;
		private double y_h1;
		
		private int x_h2 = 0;
		private double y_h2;

		private int x_hb = 0;
		private double y_hb;
				
		private double m_h;
		private double n_h;

		// Unterstützungslinie
		private int x_l1 = 0;
		private double y_l1;
		
		private int x_l2 = 0;
		private double y_l2;

		private int x_lb = 0;
		private double y_lb;
				
		private double m_l;
		private double n_l;
		
		protected override void OnStateChange() {
			
			if (State == State.SetDefaults)	{
				
				Description					= "TrendLineOne";
				Name						= "TrendLineOne";
				IsAutoScale					= false;
				IsOverlay					= true;
				IsSuspendedWhileInactive	= true;
				
				HighNum						= 10;
				LowNum						= 10;

				AddPlot( Brushes.Red, "Resistance" );
				AddPlot( Brushes.Green, "Support" );
				AddPlot( Brushes.Blue, "Resistance_Broken" );
				AddPlot( Brushes.Blue, "Support_Broken" );
				AddPlot( Brushes.Blue, "Resistance_Slope" );
				AddPlot( Brushes.Blue, "Support_Slope" );
				
			} else if (State == State.DataLoaded) {
			}
		}

		/* 
		Es werden eine aktuelle Widerstandslinie oberhalb des Kursverlaufes, sowie eine
		Unterstützungslinie unterhalb des Kursverlaufes gesucht.
		
		Für die Widerstandslinie werden zwei Punkte (H1 und H2) im Chart ermittelt, durch 
		die einie Linie gezeichnet wird. Jeder dieser Punkte hat einen X und einen Y Wert. 
		X ist die Nummer der Kerze (CurrentBar) und Y ist der Kurswert. 
		Die Punkte werden aus allen	Hochkursen der Kerzen gesucht.
		Ein Hochkurs kommt als Widerstandslinienpunkt in Frage, wenn ausreichend viele
		Hochkurse davor und danach kleiner sind. */
		protected override void OnBarUpdate() {
				
			Text t;
			
			// wenn ausreichend Kerzen vorhanden sind UND
			// wenn das Hoch vor HighNum Kerzen ein Maximum im Bereich davor und dahinter war
			if ( CurrentBar > 2*HighNum+1 && High[HighNum] == MAX( High, 2*HighNum+1 )[0] ) {
				
				// der X Wert von H2
				int new_x_h2 = CurrentBar - HighNum;
				
				// liegt H2 ausreichend weit weg vom alten H2 (der dann ja H1 werden würde)
				if ( new_x_h2 - x_h2 > HighNum  ) {
					
					// aus H2 wird H1
					y_h1 = y_h2;
					x_h1 = x_h2;
					
					// der neue Punkt H2
					y_h2 = High[HighNum];
					x_h2 = new_x_h2;
					
					// Bruch der Linie zurücksetzen
					x_hb = 0;
					y_hb = 0.0;
					RemoveDrawObject( "Punkt Bruch Widerstand" );

					t = Draw.Text( this, "H1", y_h1 + "\nH1", CurrentBar-x_h1, y_h1 );
					t.YPixelOffset = 20;
					
					Draw.Dot( this, "Punkt H1", true, CurrentBar-x_h1, y_h1, Plots[0].Brush );
					
					t = Draw.Text( this, "H2", y_h2 + "\nH2", CurrentBar-x_h2, y_h2 );
					t.YPixelOffset = 20;
					
					Draw.Dot( this, "Punkt H2", true, CurrentBar-x_h2, y_h2, Plots[0].Brush );
				}
			}
			
			// wurde inzwischen ein H1 und ein H2 gefunden?
			if ( x_h1 > 0 && x_h2 > 0 ) {
				
				// Geradengleichung
				// y = mx + n
				m_h = ( y_h2 - y_h1 ) / ( x_h2 - x_h1 );
				n_h = y_h1 - m_h * x_h1;
				
				Resistance[0] = m_h * CurrentBar + n_h;
				Resistance_Slope[0] = m_h;
				
				if ( x_hb == 0 ) {
					// Prüfung, ob Linie gebrochen wurde
					if ( High[0] > m_h * CurrentBar + n_h ) {
						Draw.Dot( this, "Punkt Bruch Widerstand", true, 0, m_h * CurrentBar + n_h, Plots[0].Brush );
						x_hb = CurrentBar;
						y_hb = m_h * CurrentBar + n_h;
					}
				}
			}
			
			Resistance_Broken[0] = x_hb > 0 ? 1.0 : 0.0;

			if ( CurrentBar > 2*LowNum+1 && Low[LowNum] == MIN( Low, 2*LowNum+1 )[0] ) {
				
				int new_x_l2 = CurrentBar - LowNum;
				
				if ( new_x_l2 - x_l2 > LowNum  ) {
					
					x_l1 = x_l2;
					y_l1 = y_l2;
					
					x_l2 = new_x_l2;
					y_l2 = Low[LowNum];
										
					x_lb = 0;
					y_lb = 0.0;
					RemoveDrawObject( "Punkt Bruch Unterstützung" );
					
					t = Draw.Text( this, "L1", "L1\n" + y_l1, CurrentBar-x_l1, y_l1 );
					t.YPixelOffset = -20;
					
					Draw.Dot( this, "Punkt L1", true, CurrentBar-x_l1, y_l1, Plots[1].Brush );
					
					t = Draw.Text( this, "L2", "L2\n" + y_l2, CurrentBar-x_l2, y_l2 );
					t.YPixelOffset = -20;
					
					Draw.Dot( this, "Punkt L2", true, CurrentBar-x_l2, y_l2, Plots[1].Brush );
				}
			}
			
			if ( x_l1 > 0 && x_l2 > 0 ) {
				
				m_l = ( y_l2 - y_l1 ) / ( x_l2 - x_l1 );
				n_l = y_l1 - m_l * x_l1;
				
				Support[0] = m_l * CurrentBar + n_l;
				Support_Slope[0] = m_l;
				
				if ( x_lb == 0 ) {
					// Prüfung, ob Linie gebrochen wurde
					if ( Low[0] < m_l * CurrentBar + n_l ) {
						Draw.Dot( this, "Punkt Bruch Unterstützung", true, 0, m_l * CurrentBar + n_l, Plots[1].Brush );
						x_lb = CurrentBar;
						y_lb = m_l * CurrentBar + n_l;
					}
				}
			}
			
			Support_Broken[0] = x_lb > 0 ? 1.0 : 0.0;
		}

		protected override void OnRender( ChartControl chartControl, ChartScale chartScale ) {
			
			Vector2 v_h1 = new Vector2( ChartControl.GetXByBarIndex( ChartBars, x_h1 ), chartScale.GetYByValue( y_h1 ) );
			Vector2 v_h2 = new Vector2( ChartControl.GetXByBarIndex( ChartBars, x_hb ), chartScale.GetYByValue( y_hb ) );
			Vector2 v_h3 = new Vector2( ChartControl.GetXByBarIndex( ChartBars, CurrentBar+10 ), chartScale.GetYByValue( m_h * (CurrentBar+10) + n_h ) );

			Vector2 v_l1 = new Vector2( ChartControl.GetXByBarIndex( ChartBars, x_l1 ), chartScale.GetYByValue( y_l1 ) );
			Vector2 v_l2 = new Vector2( ChartControl.GetXByBarIndex( ChartBars, x_lb ), chartScale.GetYByValue( y_lb ) );
			Vector2 v_l3 = new Vector2( ChartControl.GetXByBarIndex( ChartBars, CurrentBar+10 ), chartScale.GetYByValue( m_l * (CurrentBar+10) + n_l ) );
		
			RenderTarget.AntialiasMode = AntialiasMode.PerPrimitive;
			
			if ( x_hb > 0 ) {
				RenderTarget.DrawLine( v_h1, v_h2, Plots[0].BrushDX, Plots[0].Width+1, Plots[0].StrokeStyle );
				RenderTarget.DrawLine( v_h2, v_h3, Plots[0].BrushDX, Plots[0].Width, Plots[0].StrokeStyle );
			} else {
				RenderTarget.DrawLine( v_h1, v_h3, Plots[0].BrushDX, Plots[0].Width+1, Plots[0].StrokeStyle );
			}
			
			if ( x_lb > 0 ) {
				RenderTarget.DrawLine( v_l1, v_l2, Plots[1].BrushDX, Plots[1].Width+1, Plots[1].StrokeStyle );
				RenderTarget.DrawLine( v_l2, v_l3, Plots[1].BrushDX, Plots[1].Width, Plots[1].StrokeStyle );
			} else {
				RenderTarget.DrawLine( v_l1, v_l3, Plots[1].BrushDX, Plots[1].Width+1, Plots[1].StrokeStyle );
			}
			
			RenderTarget.AntialiasMode = AntialiasMode.Aliased;
		}
		
		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Resistance { get { return Values[0]; }	}

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Support { get { return Values[1]; } }
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Resistance_Broken { get { return Values[2]; } }

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Support_Broken { get { return Values[3]; } }

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Resistance_Slope { get { return Values[4]; } }
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Support_Slope { get { return Values[5]; } }


		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Abstand Hoch", GroupName = "NinjaScriptGeneral", Order = 0)]
		public int HighNum
		{ get; set; }

		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Abstand Tief", GroupName = "NinjaScriptGeneral", Order = 1)]
		public int LowNum
		{ get; set; }
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private TrendLineOne[] cacheTrendLineOne;
		public TrendLineOne TrendLineOne(int highNum, int lowNum)
		{
			return TrendLineOne(Input, highNum, lowNum);
		}

		public TrendLineOne TrendLineOne(ISeries<double> input, int highNum, int lowNum)
		{
			if (cacheTrendLineOne != null)
				for (int idx = 0; idx < cacheTrendLineOne.Length; idx++)
					if (cacheTrendLineOne[idx] != null && cacheTrendLineOne[idx].HighNum == highNum && cacheTrendLineOne[idx].LowNum == lowNum && cacheTrendLineOne[idx].EqualsInput(input))
						return cacheTrendLineOne[idx];
			return CacheIndicator<TrendLineOne>(new TrendLineOne(){ HighNum = highNum, LowNum = lowNum }, input, ref cacheTrendLineOne);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.TrendLineOne TrendLineOne(int highNum, int lowNum)
		{
			return indicator.TrendLineOne(Input, highNum, lowNum);
		}

		public Indicators.TrendLineOne TrendLineOne(ISeries<double> input , int highNum, int lowNum)
		{
			return indicator.TrendLineOne(input, highNum, lowNum);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.TrendLineOne TrendLineOne(int highNum, int lowNum)
		{
			return indicator.TrendLineOne(Input, highNum, lowNum);
		}

		public Indicators.TrendLineOne TrendLineOne(ISeries<double> input , int highNum, int lowNum)
		{
			return indicator.TrendLineOne(input, highNum, lowNum);
		}
	}
}

#endregion
